#include <Windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "..\Library\rdsapi.h"

long ProfileID = 0;

// Function called when profile data is received
void profile_data(void* arg, const rdsProfileInfo* item)
{
  printf("Profile:\n");
  printf("-id=         %d\n", item->id_number);
  printf("-name=       [%s]\n", item->name);
  printf("-password=   [%s]\n", item->password);
  printf("-flags=      0x%08X\n", item->flags);
  printf("-Start time= %d\n", item->time_start);
  printf("-Start time= %d\n", item->time_stop);
  printf("-app_name=   [%s]\n", item->app_name);
  printf("-app_ver=    %d\n", item->app_ver);
  printf("-app_file=   [%s]\n", item->app_file);
  printf("-fw_tgt=     %d\n", item->fw_tgt);
  printf("-fw_ver=     %d\n", item->fw_system);
  printf("-fw_file=    [%s]\n", item->fw_file);
  printf("-file_file=  [%s]\n", item->file_file);
  printf("-file_dst=   [%s]\n", item->file_dst);
  ProfileID = item->id_number;
}

// Function called when unit data is received
void unit_data(void* arg, const rdsUnitInfo* item)
{
  printf("Unit:\n");
  printf("-serial=     %d\n", item->serial_number);
  printf("-profile=    %d\n", item->profile_id);
  printf("-password=   [%s]\n", item->password);
  printf("-flags=      0x%08X\n", item->flags);
  printf("-app_name=   [%s]\n", item->app_name);
  printf("-app_ver=    %d\n", item->app_ver);
  printf("-fw_tgt=     %d\n", item->fw_tgt);
  printf("-fw_ver=     %d\n", item->fw_system);
  printf("-progress=   %d\n", item->progress);
  printf("-gateway=    %d\n", item->gateway);
  printf("-status=     %d\n", item->status);
  printf("-timestamp=  %ld\n", item->timestamp);
  printf("-comment=    [%s]\n", item->comment);
}

int main(int argc, char* argv[])
{
  // Var
  unsigned short gw_port;
  int            rc;
  rdsProfileInfo profile;

  // Write welcome
  printf("----------------------------------------\n");
  printf("  RTCU Deployment Server API demo\n");
  printf("----------------------------------------\n");

  // Test arguments
  if (argc != 5) {
    printf("Parameters not correct.\n");
    printf("\nToo use tool:\n");
    printf("rdsDemo <ip addr> <ip port> <gw key> <rds key>\n");
    printf("\nFor example: rdsDemo gw.logicio.com 5001 AABBCCDD RDS\n");
    printf("----------------------------------------\n");
    return 1;
  }

  // Report config
  printf("RDSAPI version: %d.%02d\n", rdsGetVersion() / 100, rdsGetVersion() % 100);
  printf("gateway addr:   [%s]\n", argv[1]);
  printf("gateway port:   [%s]\n", argv[2]);
  printf("gateway key:    [%s]\n", argv[3]);
  printf("RDS key:        [%s]\n", argv[4]);
  printf("----------------------------------------\n");

  // Config library
  rdsRegisterProfileNotification(nullptr, profile_data, nullptr);
  rdsRegisterUnitNotification(nullptr, unit_data, nullptr);

  // Open connection
  gw_port = atoi(argv[2]);
  rc      = rdsConnect(argv[1], gw_port, argv[3], argv[4]);
  if (rc) {
    printf(" Illegal parameter!\n");
    return 1;
  }

  // Wait for connection
  printf(" Connecting to gateway.");
  while (rdsGetStatus() < 2) { Sleep(500); printf("."); }
  printf("done\n");
  printf(" Connecting to RDS.");
  while (rdsGetStatus() < 4) { Sleep(500); printf("."); }
  if (rdsGetStatus() > 4) {
    printf("error\n");
    switch (rdsGetStatus()) {
      case 12: printf("RDS not found!\n"); break;
      case 13: printf("Wait.. Another client is already connected to RDS.\n"); break;
      case 14: printf("Incorrect RDS login password!\n"); break;
      case 15: if (rdsGetServerVersion() < 300) {
                 printf("UDS server %d.%02d is not supported!\n", rdsGetServerVersion() / 100, rdsGetServerVersion() % 100); break;
               } else {
                 printf("RDS server %d.%02d is not supported!\n", rdsGetServerVersion() / 100, rdsGetServerVersion() % 100); break;
               }
      default: printf("Unknown error %d\n", rdsGetStatus()); break;
    }
    rdsDisconnect();
    return 1;
  }
  printf("done\n");

  // Create profile
  printf(" Create profile...");
  profile.id_number = 0;      //Set by the RDS
  strcpy_s(profile.name, 65, "Test profile");
  strcpy_s(profile.password, 21, "");
  profile.flags      = PFLAG_TARGETDEFAULT|PFLAG_APP_ENABLE|PFLAG_FW_ENABLE;
  profile.time_start = -1;
  profile.time_stop  = -1;
  strcpy_s(profile.app_name, 16, "TestApp");
  profile.app_ver    = 100;
  strcpy_s(profile.app_file, 191, "testapp.vsx");
  profile.fw_tgt     = 102;
  profile.fw_system  = 280;
  strcpy_s(profile.fw_file, 191, "Firmware_MX2_280.bin");
  strcpy_s(profile.file_file, 191, "");
  strcpy_s(profile.file_dst, 61, "A:");
  rc = rdsProfileCreate(&profile);
  if (rc) {
    printf("error\n");
    switch (rdsGetStatus()) {
      case -3: printf("Communication error.\n"); break;
      case -2: printf("RDS server not connected to gateway.\n"); break;
      case -1: printf("Not connected to gateway.\n"); break;
      case 1:  printf("General error.\n"); break;
      case 2:  printf("Other profile with the same name.\n"); break;
      case 3:  printf("Other profile is set as default for target.\n"); break;
      case 4:  printf("Invalid file parameter.\n"); break;
      case 5:  printf("Invalid application parameter.\n"); break;
      case 6:  printf("Invalid firmware parameter.\n"); break;
      case 7:  printf("Invalid time interval.\n"); break;
      case 8:  printf("Name is missing.\n"); break;
      case 9:  printf("No Application, Firmware and File is selected.\n"); break;
      default: printf("Unknown error %d\n", rc); break;
    }
    rdsDisconnect();
    return 1;
  }
  printf("done\n");
  Sleep(500);

  // Create unit
  printf(" Create unit...");
  rc = rdsUnitCreate(219105667, ProfileID, 0, "", "rdsDemo unit");
  if (rc) {
    printf("error\n");
    switch (rdsGetStatus()) {
      case -3: printf("Communication error.\n"); break;
      case -2: printf("RDS server not connected to gateway.\n"); break;
      case -1: printf("Not connected to gateway.\n"); break;
      case 1:  printf("General error.\n"); break;
      case 2:  printf("Other unit exists.\n"); break;
      case 3:  printf("Invalid parameter.\n"); break;
      default: printf("Unknown error %d\n", rc); break;
    }
    rdsDisconnect();
    return 1;
  }
  printf("done\n");
  Sleep(500);

  // Disconnect
  printf(" Closing.");
  rdsDisconnect();
  while (rdsGetStatus() > 0) { Sleep(500); printf("."); }
  printf("done\n");

  // Completed
  return 0;
}